"""before publishing hook to allow domain authentication

this hook is designed for Zope >= 2.2.x
"""

from ZPublisher.BeforeTraverse import registerBeforeTraverse, queryBeforeTraverse, unregisterBeforeTraverse
from AccessControl.User import domainSpecMatch, _remote_user_mode
from base64 import decodestring
from Actions import IGNORE, MUSTAUTH, AUTHAS, AUTHASIFANON
from Rules import get_rules

HOOK_PRIORITY = 10

class BasicAuthHook:
    """before publishing hook to allow domain authentication in classic (basic)
    authentication mode
    """

    def __call__(self, container, request,
                 # 'default' arguments for speed
                 domainSpecMatch=domainSpecMatch, decodestring=decodestring,
                 IGNORE = IGNORE, MUSTAUTH = MUSTAUTH,
                 AUTHAS = AUTHAS, AUTHASIFANON = AUTHASIFANON):
        rules = get_rules()

        if rules:
            for id_r, domains_r, id_a, name_r, auth_r, excepts_r in rules:
                if domainSpecMatch(domains_r, request):
                    if excepts_r and request._auth:
                        auth = request._auth
                        if lower(auth[:6]) == 'basic ':
                            # we only do basic authentication
                            name, password = split(decodestring(split(auth)[-1]), ':', 1)
                            if name in excepts_r:
                                # ignore this user for this action
                                continue
                    if id_a == IGNORE:
                        break
                    elif id_a == MUSTAUTH:
                        if not request._auth:
                            request.response.unauthorized()
                        break
                    elif id_a == AUTHAS:
                        request._auth = auth_r
                        break
                    elif id_a == AUTHASIFANON and not request._auth:
                        request._auth = auth_r
                        break
            else:
                # If no rules apply, let Zope deal with the users its way
                # XXX Is this what we really want?
                pass
        else:
            # 'classic' behavior: if no rules are defined, force authentication of users
            if not request._auth:
                request.response.unauthorized()

class RemoteUserModeHook:
    """before publishing hook to allow domain authentication in remote user
    authentication mode
    """

    def __call__(self, container, request,
                 # 'default' arguments for speed
                 domainSpecMatch=domainSpecMatch,
                 IGNORE = IGNORE, MUSTAUTH = MUSTAUTH,
                 AUTHAS = AUTHAS, AUTHASIFANON = AUTHASIFANON):
        rules = get_rules()
        environ = request.environ
        remote_user = environ.get('REMOTE_USER', '')

        if rules:
            for id_r, domains_r, id_a, name_r, auth_r, excepts_r in rules:
                if domainSpecMatch(domains_r, request):
                    if excepts_r and remote_user and \
                       remote_user in excepts_r:
                        # ignore this user for this action
                        continue
                    if id_a == IGNORE:
                        break
                    elif id_a == MUSTAUTH:
                        if not remote_user:
                            request.response.unauthorized()
                        break
                    elif id_a == AUTHAS:
                        environ['REMOTE_USER'] = name_r
                        break
                    elif id_a == AUTHASIFANON and not remote_user:
                        environ['REMOTE_USER'] = name_r
                        break
            else:
                # If no rules apply, let Zope deal with the users its way
                # XXX Is this what we really want?
                pass
        else:
            # 'classic' behavior: if no rules are defined, force authentication of users
            if not remote_user:
                request.response.unauthorized()

# install hook
def install(app):
    # get the application object from the context
    if _remote_user_mode:
        klass = RemoteUserModeHook
    else:
        klass = BasicAuthHook
        
    hook = queryBeforeTraverse(app, 'jcForceAuth')[0][1]
    if not isinstance(hook, klass):
        registerBeforeTraverse(app, klass(), 'jcForceAuth', HOOK_PRIORITY)
        get_transaction().note('[registered hook]')
    return 1

def uninstall(app):
    # get the application object from the context
    app = context._ProductContext__app
    if queryBeforeTraverse(app, 'jcForceAuth'):
        unregisterBeforeTraverse(app, 'jcForceAuth')
        get_transaction().note('[unregistered hook]')
    return 1
