"""patch to allow domain authentication

this patch is designed for Zope 2.1.x. Later releases no longer need to be
patched (they have before publishing hooks)
"""

from AccessControl.User import domainSpecMatch, _remote_user_mode
from base64 import decodestring
from Actions import IGNORE, MUSTAUTH, AUTHAS, AUTHASIFANON
from Rules import get_rules

def traverse_basic(self, path, response=None,
                   # 'default' arguments for speed
                   domainSpecMatch=domainSpecMatch, decodestring=decodestring,
                   IGNORE = IGNORE, MUSTAUTH = MUSTAUTH,
                   AUTHAS = AUTHAS, AUTHASIFANON = AUTHASIFANON,
                   **kw):
    """traverse() replacement for classic (basic) authentication mode"""
    rules = get_rules()

    if rules:
        for id_r, domains_r, id_a, name_r, auth_r, excepts_r in rules:
            if domainSpecMatch(domains_r, self):
                if excepts_r and self._auth:
                    auth = self._auth
                    if lower(auth[:6]) == 'basic ':
                        # we only do basic authentication
                        name, password = split(decodestring(split(auth)[-1]), ':', 1)
                        if name in excepts_r:
                            # ignore this user for this action
                            continue
                if id_a == IGNORE:
                    break
                elif id_a == MUSTAUTH:
                    if response is None:
                        response = self.response
                    if not self._auth:
                        response.unauthorized()
                    break
                elif id_a == AUTHAS:
                    self._auth = auth_r
                    break
                elif id_a == AUTHASIFANON and not self._auth:
                    self._auth = auth_r
                    break
        else:
            # If no rules apply, let Zope deal with the users its way
            # XXX Is this what we really want?
            pass
    else:
        # 'classic' behavior: if no rules are defined, force authentication of users
        if response is None:
            response = self.response
        if not self._auth:
            response.unauthorized()

    # restore the old method for subsequent call to traverse not be too expensive
    _old_traverse = self._old_traverse
    self.traverse = _old_traverse
    return apply(_old_traverse, (path, response), kw)

def traverse_remote(self, path, response=None,
                    # 'default' arguments for speed
                    domainSpecMatch=domainSpecMatch,
                    IGNORE = IGNORE, MUSTAUTH = MUSTAUTH,
                    AUTHAS = AUTHAS, AUTHASIFANON = AUTHASIFANON,
                    **kw):
    """traverse() replacement for remote user mode"""
    rules = get_rules()
    environ = self.environ
    remote_user = environ.get('REMOTE_USER', '')

    if rules:
        for id_r, domains_r, id_a, name_r, auth_r, excepts_r in rules:
            if domainSpecMatch(domains_r, self):
                if excepts_r and remote_user and \
                   remote_user in excepts_r:
                    # ignore this user for this action
                    continue
                if id_a == IGNORE:
                    break
                elif id_a == MUSTAUTH:
                    if response is None:
                        response = self.response
                    if not remote_user:
                        response.unauthorized()
                    break
                elif id_a == AUTHAS:
                    environ['REMOTE_USER'] = name_r
                    break
                elif id_a == AUTHASIFANON and not remote_user:
                    environ['REMOTE_USER'] = name_r
                    break
        else:
            # If no rules apply, let Zope deal with the users its way
            # XXX Is this what we really want?
            pass
    else:
        # 'classic' behavior: if no rules are defined, force authentication of users
        if response is None:
            response = self.response
        if not remote_user:
            response.unauthorized()

    # restore the old method for subsequent call to traverse not be too expensive
    _old_traverse = self._old_traverse
    self.traverse = _old_traverse
    return apply(_old_traverse, (path, response), kw)

# apply patch
def install():
    from ZPublisher.BaseRequest import BaseRequest
    dict = BaseRequest.__dict__
    dict['_old_traverse'] = dict['traverse']
    if _remote_user_mode:
        dict['traverse'] = traverse_remote
    else:
        dict['traverse'] = traverse_basic
    return 1 # install avec succs

def uninstall():
    from ZPublisher.BaseRequest import BaseRequest
    dict = BaseRequest.__dict__
    if dict.has_key('_old_traverse'):
        dict['traverse'] = dict['_old_traverse']
        del dict['_old_traverse']
    return 1 # dsinstall avec succs
