from Zope import DB

# avoid loading the rules on each request
_rules_cache = None

class Rule:
    """Rule object for use from DTML
    """
    
    def __init__(self, rule):
        """Initialize object

        rule is a tuple (id_r, domains_r, id_a, name_r, auth_r, excepts_r)
        """
        (self.id_r, self.domains_r, self.id_a,
         self.name_r, self.auth_r, self.excepts_r) = rule

def load_rules(cooked=0):
    """Load the rules from the ZODB

    Each rule is a tuple (id_r, domains_r, id_a, name_r, auth_r, excepts_r) in
    the ZODB. If cooked==1, the rules are returned as Rule instances instead.
    """
    conn = DB.open()
    root = conn.root()
    rules = root.get('_jcFA_rules', ())
    conn.close()
    if cooked: rules = map(Rule, rules)
    return rules

def save_rules(rules):
    """Write back the rules in the ZODB

    rules is a sequence of rule tuples (id_r, domains_r, id_a, name_r, auth_r,
    excepts_r)
    """
    rules = tuple(rules)
    conn = DB.open()
    root = conn.root()
    root['_jcFA_rules'] = rules
    get_transaction().note('wrote the new rules')
    get_transaction().commit()
    conn.close()

def get_rules():
    """Get the rules, loading them from the ZODB if necessary
    """
    global _rules_cache
    if _rules_cache is None:
        _rules_cache = load_rules()
    return _rules_cache

def apply_rules():
    """Empty the rules cache, to force them to be reloaded from the ZODB
    """
    global _rules_cache
    _rules_cache = None
